import { Button, message, Tooltip } from 'antd'
import type { ActionType } from '@ant-design/pro-table'
import ProTable from '@ant-design/pro-table'
import React, { useState, useRef } from 'react'
import { useRequest } from 'umi'
import {
  ArrowDownOutlined,
  ArrowUpOutlined,
  DeleteOutlined,
  EditOutlined,
  PlusSquareOutlined,
  SwapOutlined
} from '@ant-design/icons'
import {
  queryOrganizationalStructureList,
  moveSortOrganizationalStructure
} from '@/services/api/institution'
import OrganizationModal from './OrganizationModal'
// import { ModalType } from '@/utils/enum'

type OrganizationProps = {
  dataID: string
  structureType: string
}

export enum ModalTypeEnum {
  ADD = 1,
  ADDITEM = 2,
  UPDATE = 3,
  DEL = 4,
  MOVE = 5
}

const Organization: React.FC<OrganizationProps> = ({ dataID, structureType }) => {
  const tRef = useRef<ActionType>(null)
  const [state, setState] = useState({
    organizationList: [],
    visible: false,
    expandTreeIds: [],
    defaultFormData: null,
    parentID: '',
    disableID: '',
    currentModalType: ModalTypeEnum.ADD,
    accountTotal: null
  })

  const { run: tryOrganizationList } = useRequest(
    () => queryOrganizationalStructureList({ dataID, structureType }),
    {
      // manual: true,
      onSuccess: result => {
        function expandTreeList(data: OrganizationalStructureListItem[]): string[] {
          let idArr: string[] = []
          data.forEach((item: OrganizationalStructureListItem) => {
            if (item.children?.length) {
              idArr.push(item.ID)
              const childIDs = expandTreeList(item.children)
              idArr = [...idArr, ...childIDs]
            }
          })
          return idArr
        }
        setState({
          ...state,
          organizationList: result,
          expandTreeIds: expandTreeList(result)
        })
      }
    }
  )

  const { run: tryMoveOrganization } = useRequest(moveSortOrganizationalStructure, {
    manual: true,
    onSuccess: () => {
      tryOrganizationList()
      message.success('移动成功')
    }
  })

  const columns: ColumnType<API.OrganizationalStructureListItem>[] = [
    {
      title: '组织名称',
      dataIndex: 'name',
      key: 'name'
    },
    {
      title: '包含人员',
      dataIndex: 'accountTotal',
      key: 'accountTotal'
    },
    {
      title: '操作',
      dataIndex: 'operation',
      key: 'operation',
      render: (_, record) => (
        <div className="divide-x divide-bg-gray-400 flex flex-row">
          {record.isStart === false ? (
            <div
              className="px-2 text-primary cursor-pointer hover:text-hex-967bbd"
              onClick={() => tryMoveOrganization({ ID: record.ID, operation: 'up' })}>
              <Tooltip placement="top" title="上移">
                <ArrowUpOutlined />
              </Tooltip>
            </div>
          ) : null}
          {record.isEnd === false ? (
            <div
              className="px-2 text-primary cursor-pointer hover:text-hex-967bbd"
              onClick={() => tryMoveOrganization({ ID: record.ID, operation: 'down' })}>
              <Tooltip placement="top" title="下移">
                <ArrowDownOutlined />
              </Tooltip>
            </div>
          ) : null}
          {record.isLeaf === true ? (
            <div
              className="px-2 text-primary cursor-pointer hover:text-hex-967bbd"
              onClick={() => {
                setState({
                  ...state,
                  visible: true,
                  defaultFormData: record,
                  currentModalType: ModalTypeEnum.MOVE
                })
              }}>
              <Tooltip placement="top" title="移动">
                <SwapOutlined />
              </Tooltip>
            </div>
          ) : null}
          <div
            className="px-2 text-primary cursor-pointer hover:text-hex-967bbd"
            onClick={() => {
              // console.log(record.parentID)
              setState({
                ...state,
                visible: true,
                defaultFormData: {
                  dataID: dataID,
                  structureType: structureType,
                  parentID: record.ID
                },
                currentModalType: ModalTypeEnum.ADDITEM
              })
            }}>
            <Tooltip placement="top" title="添加子项">
              <PlusSquareOutlined />
            </Tooltip>
          </div>
          <div
            className="px-2 text-primary cursor-pointer hover:text-hex-967bbd"
            onClick={() => {
              setState({
                ...state,
                visible: true,
                defaultFormData: record,
                currentModalType: ModalTypeEnum.UPDATE
              })
            }}>
            <Tooltip placement="top" title="重命名">
              <EditOutlined />
            </Tooltip>
          </div>
          {!record.parentID === false ? (
            <div className="pl-2 text-hex-fd3995 cursor-pointer hover:text-hex-967bbd">
              <Tooltip placement="top" title="删除">
                <DeleteOutlined
                  onClick={() => {
                    setState({
                      ...state,
                      visible: true,
                      disableID: record.ID,
                      defaultFormData: record,
                      currentModalType: ModalTypeEnum.DEL,
                      accountTotal: record.accountTotal
                    })
                  }}
                />
              </Tooltip>
            </div>
          ) : null}
        </div>
      )
    }
  ]

  return (
    <div>
      <ProTable<API.OrganizationalStructureListItem>
        rowKey="ID"
        actionRef={tRef}
        columns={columns}
        dataSource={state.organizationList}
        // defaultExpandAllRows={true}
        expandable={{
          expandedRowKeys: state.expandTreeIds,
          onExpand: (expanded, record) => {
            if (expanded) {
              if (!state.expandTreeIds.includes(record.ID)) {
                return setState({ ...state, expandTreeIds: [...state.expandTreeIds, record.ID] })
              }
            } else {
              setState({
                ...state,
                expandTreeIds: state.expandTreeIds.filter(item => item !== record.ID)
              })
            }
          }
        }}
        // expandedRowRender={expandedRowRender}
        indentSize={30}
        search={false}
        pagination={false}
        toolbar={{
          actions: [
            <Button
              type="primary"
              onClick={() =>
                setState({
                  ...state,
                  visible: true,
                  currentModalType: ModalTypeEnum.ADD,
                  defaultFormData: {
                    dataID: dataID,
                    structureType: structureType,
                    parentID: state.parentID
                  }
                })
              }>
              添加组织
            </Button>
          ]
        }}
      />
      <OrganizationModal
        type={state.currentModalType}
        visible={state.visible}
        disableID={state.disableID}
        setVisible={(visible: boolean) => setState({ ...state, visible })}
        reload={() => {
          tryOrganizationList()
        }}
        defaultFormData={state.defaultFormData}
        dataID={dataID}
        structureType={structureType}
        accountTotal={state.accountTotal}
        treeSource={state.organizationList}
      />
    </div>
  )
}

export default Organization
