import { useRequest } from 'umi'
import React, { useRef, useEffect, useState } from 'react'
import { Modal, message, TreeSelect, Form } from 'antd'
import type { FormInstance } from 'antd'
import ProForm, { ProFormText } from '@ant-design/pro-form'
import {
  addOrganizationalStructure,
  updateOrganizationalStructure,
  delOrganizationalStructure,
  moveOrganizationalStructure,
  queryOrganizationalStructureList
} from '@/services/api/institution'
import { ModalTypeEnum } from './Organization'

type OrganizationModalProps = {
  visible: boolean
  setVisible: (visible: boolean) => void
  type: ModalTypeEnum
  reload: () => void
  defaultFormData?: {
    structureType?: string
    dataID?: string
    name?: string
    parentID?: string
  }
  treeSource: string
  dataID: string
  disableID: string
  structureType: string
  accountTotal: number
}
const OrganizationModal: React.FC<OrganizationModalProps> = ({
  visible,
  type,
  dataID,
  structureType,
  setVisible,
  defaultFormData,
  treeSource,
  accountTotal,
  disableID,
  reload
}) => {
  const ref = useRef<FormInstance>(null)
  const [state, setState] = useState({
    organizationList: []
  })

  const titleTypeEunm = {
    1: {
      key: 'ADD',
      name: '添加父级组织'
    },
    2: {
      key: 'ADDITEM',
      name: '添加子项'
    },
    3: {
      key: 'UPDATE',
      name: '重命名'
    },
    4: {
      key: 'DEL',
      name: '删除组织架构'
    },
    5: {
      key: 'MOVE',
      name: '移动'
    }
  }
  const { run: tryOrganizationList } = useRequest(
    () => queryOrganizationalStructureList({ dataID, structureType }),
    {
      manual: true,
      onSuccess: result => {
        setState({ ...state, organizationList: result })
      }
    }
  )
  const { run: tryUpdateOrganization } = useRequest(updateOrganizationalStructure, {
    manual: true,
    onSuccess: () => {
      message.success('更新成功')
      tryOrganizationList()
    }
  })
  const { run: tryAddOrganization } = useRequest(addOrganizationalStructure, {
    manual: true,
    onSuccess: () => {
      message.success('创建成功')
      tryOrganizationList()
    }
  })
  const { run: tryDelOrganization } = useRequest(delOrganizationalStructure, {
    manual: true,
    onSuccess: () => {
      message.success('删除成功')
      tryOrganizationList()
    }
  })
  const { run: tryMoveOrganization } = useRequest(moveOrganizationalStructure, {
    manual: true,
    onSuccess: () => {
      message.success('移动成功')
      tryOrganizationList()
    }
  })
  useEffect(() => {
    // tryOrganizationList()
    // if (!state.organizationList?.length) {
    //   tryOrganizationList()
    // }
    visible && defaultFormData && ref.current?.setFieldsValue({ ...defaultFormData })
  }, [defaultFormData, visible])

  const handleOnFinish = () => {
    ref.current?.validateFields().then(async values => {
      try {
        // 执行表单提交
        if (type === ModalTypeEnum.ADD || type === ModalTypeEnum.ADDITEM) {
          await tryAddOrganization({ ...values, ...defaultFormData })
        } else if (type === ModalTypeEnum.DEL) {
          await tryDelOrganization(values)
        } else if (type === ModalTypeEnum.MOVE) {
          await tryMoveOrganization(values)
        } else {
          await tryUpdateOrganization(values)
        }

        setVisible(false)

        ref.current?.resetFields()
        reload()
      } catch (error) {
        message.error(error)
      }
    })
  }

  const onChange = () => {
    if (!state.organizationList?.length) {
      tryOrganizationList()
    }
  }

  const onNodeChange = (value, labelList) => {
    onChange(value, { value, label: labelList[0] })
  }
  const renderTreeNodes = data =>
    data.map(item => {
      const newItem = { ...item }
      if (item.ID === disableID) {
        // console.log('我是ID')
        newItem.disabled = true
      }
      if (newItem.children) {
        // newItem.disabled = true
        newItem.children = renderTreeNodes(newItem.children)
        return newItem
      }
      return newItem
    })

  return (
    <Modal
      width="35vw"
      visible={visible}
      // title={type === ModalType.ADD ? '添加父级组织' : '重命名'}
      title={titleTypeEunm[type].name}
      onCancel={() => {
        ref.current?.resetFields()
        setVisible(false)
      }}
      onOk={() => handleOnFinish()}>
      <ProForm formRef={ref} submitter={{ render: false }}>
        {type === ModalTypeEnum.ADD || type === ModalTypeEnum.ADDITEM ? (
          <>
            <ProFormText
              name="name"
              label="组织架构名称"
              rules={[{ required: true, message: '请输入组织架构名称' }]}
            />
          </>
        ) : null}
        {type === ModalTypeEnum.UPDATE ? (
          <>
            <ProFormText name="ID" hidden />
            <ProFormText
              name="name"
              label="组织架构名称"
              rules={[{ required: true, message: '请输入组织架构名称' }]}
            />
          </>
        ) : null}
        {type === ModalTypeEnum.DEL ? (
          <>
            <ProFormText name="ID" hidden />
            <p>确认删除？</p>
            {accountTotal ? (
              <div>
                <p>请设置包含人员至新的组织架构中：</p>
                <Form.Item label="目标架构" name="moveID">
                  {treeSource?.length && (
                    <TreeSelect
                      style={{ width: '100%' }}
                      placeholder="请选择目标架构"
                      treeDefaultExpandAll
                      treeNodeFilterProp="label"
                      onChange={(value, labelList, extra) => onNodeChange(value, labelList, extra)}
                      treeData={renderTreeNodes(treeSource)}
                    />
                  )}
                  {/* <TreeSelect
                    style={{ width: '100%' }}
                    placeholder="请选择目标架构"
                    treeDefaultExpandAll
                    treeData={treeSource}
                  /> */}
                </Form.Item>
              </div>
            ) : null}
          </>
        ) : null}
        {type === ModalTypeEnum.MOVE ? (
          <>
            <ProFormText name="ID" hidden />
            <Form.Item label="目标架构" name="moveID">
              <TreeSelect
                style={{ width: '100%' }}
                placeholder="请选择目标架构"
                treeDefaultExpandAll
                treeData={treeSource}
              />
            </Form.Item>
          </>
        ) : null}
      </ProForm>
    </Modal>
  )
}

export default OrganizationModal
