import { Select, Spin } from 'antd'
import type { SelectProps } from 'antd'
import { useRef, useState, useMemo, useEffect } from 'react'
import debounce from 'lodash/debounce'

export interface DebounceSelectProps<ValueType = any>
  extends Omit<SelectProps<ValueType>, 'options' | 'children'> {
  fetchOptions: (search: string) => Promise<ValueType[]>
  debounceTimeout?: number
  defaultOptions?: ValueType[]
}

function DebounceSelect<
  ValueType extends { key?: string; label: React.ReactNode; value: string | number } = any
>({
  fetchOptions,
  debounceTimeout = 800,
  value: val,
  onChange,
  defaultOptions,
  ...props
}: DebounceSelectProps) {
  // console.log(val)

  const [state, setState] = useState<{ val?: string; fetching: boolean; options: ValueType[] }>({
    val,
    fetching: false,
    options: defaultOptions || []
  })
  const fetchRef = useRef(0)

  const debounceFetcher = useMemo(() => {
    const loadOptions = params => {
      fetchRef.current += 1
      const fetchId = fetchRef.current
      setState({ ...state, options: [], fetching: true })

      fetchOptions(params).then(newOptions => {
        if (fetchId !== fetchRef.current) {
          // for fetch callback order
          return
        }

        setState({ ...state, options: newOptions, fetching: false })
      })
    }

    return debounce(loadOptions, debounceTimeout)
  }, [fetchOptions, debounceTimeout])
  useEffect(() => {
    if (!state.options?.length && state.val) {
      fetchOptions({ ID: state.val }).then(newOptions => {
        setState({ ...state, options: newOptions, fetching: false })
      })
    }
  }, [state.val])

  const triggerChange = v => {
    setState({ ...state, val: v })
    if (v) {
      onChange(v)
    }
    // setTimeout(() => {
    //   console.log(state.options)
    // }, 500)
  }
  // console.log(state.options)

  return (
    <Select<ValueType>
      value={state.val}
      onChange={triggerChange}
      filterOption={false}
      onSearch={v => debounceFetcher({ search: v })}
      notFoundContent={state.fetching ? <Spin size="small" /> : null}
      {...props}
      options={state.options}
    />
  )
}

export default DebounceSelect
