import React, { useEffect } from 'react'
import { message, Button } from 'antd'
import { connect, useRequest } from 'umi'
import consts from '@/utils/consts'
import {
  addAccount,
  updateAccount,
  queryAccountDetail,
  queryInstitutionList,
  queryOrganizationalStructureList
} from '@/services/api/institution'
import DebounceSelect from './DebounceSelect'
// import { delay } from '@/utils/util'
import FormRender, { useForm } from 'form-render'
import { BaseMenuEnum } from '@/pages/Schema/Base'
import type { SchemaBaseModelState } from '@/pages/Schema/Base/model'
import type { ConnectProps } from 'umi'
import type { InstitutionsModelState } from '../../model'

import 'antd/lib/tree-select/style/index'
import { ModalType } from '@/utils/enum'

type StaffModalProps = ConnectProps & {
  visible: boolean
  readOnly: boolean
  onVisibleChange: (visible: boolean) => void
  institutionDisable: boolean
  type: ModalType
  defaultFormData?: {
    dataID?: string
    institutionID?: string
    institutionDisable?: boolean
  }
  accountTypeList: API.AccountType
  organizationList: API.OrganizationalStructureListItem[]
  reload: () => void
  schema?: Record<string, any> | null
}

const StaffDrawer: React.FC<StaffModalProps> = ({
  visible,
  onVisibleChange,
  schema,
  dispatch,
  type,
  defaultFormData,
  accountTypeList,
  reload
}) => {
  const form = useForm()
  const { run: tryUpdateAccount } = useRequest(updateAccount, {
    manual: true,
    onSuccess: () => {
      message.success('更新成功')
    }
  })

  const { run: tryAddAccount } = useRequest(addAccount, {
    manual: true,
    onSuccess: () => {
      message.success('创建成功')
    }
  })

  useEffect(() => {
    if (visible) {
      if (!schema) {
        dispatch({
          type: 'schemaBase/querySchema',
          payload: {
            columnType: BaseMenuEnum.STAFF
          }
        })
      }
    }
  }, [visible])

  const queryInstitutionOptions = async params => {
    const { code = -1, data = {} } = await queryInstitutionList({
      ...params,
      current: 1,
      pageSize: 100
    })
    if (code === consts.RET_CODE.SUCCESS) {
      return data.items.map(item => ({ label: item.name, value: item.ID }))
    }
    return []
  }

  const SiteInput = props => {
    return (
      <DebounceSelect
        className="w-full"
        fetchOptions={queryInstitutionOptions}
        showSearch
        // options={institution?.map(item => ({ label: item.name, value: item.ID }))}
        {...props}
      />
    )
  }

  const onMount = async () => {
    const {
      dataID,
      institutionID,
      institutionDisable = false,
      hiddenOrganization = false
    } = defaultFormData

    if (institutionID && type !== ModalType.PREVIEW) {
      const { code = -1, data: list = [] } = await queryOrganizationalStructureList({
        dataID: institutionID,
        structureType: '1'
      })
      if (code === consts.RET_CODE.SUCCESS) {
        form.setSchemaByPath('organizationalStructureID', {
          type: 'string',
          widget: 'treeSelect',
          props: {
            treeDefaultExpandAll: true,
            treeData: list
          }
        })
      }
    }
    if (dataID) {
      const { code = -1, data = {} } = await queryAccountDetail({ ID: dataID })
      if (code === consts.RET_CODE.SUCCESS) {
        const account = {
          ...data,
          institution: null,
          organizationalStructure: null,
          institutionID: data?.institution?.ID,
          organizationalStructureID: data?.organizationalStructure?.ID
        }
        if (type === ModalType.PREVIEW) {
          account.institutionID = data.institution.name
          account.organizationalStructureID = data.organizationalStructure?.name
        }
        form.setValues({
          ...account
        })
      }
    } else {
      if (type === ModalType.ADD) {
        const values = {}
        if (institutionID) values.institutionID = institutionID
        form.setValues(values)
      }
    }
    form.setSchemaByPath('accountType', {
      type: 'string',
      widget: 'select',
      enum: accountTypeList.map(item => item.value),
      enumNames: accountTypeList.map(item => item.label)
    })
    form.setSchemaByPath('institutionID', {
      type: 'string',
      widget: 'site',
      disabled: institutionDisable
    })
    if (type === ModalType.UPDATE) {
      form.setSchemaByPath('account', {
        disabled: true
      })
    }

    form.setSchemaByPath('organizationalStructureID', {
      disabled: hiddenOrganization,
      hidden: !institutionID
    })
    if (hiddenOrganization && (type === ModalType.UPDATE || type === ModalType.PREVIEW)) {
      form.setSchemaByPath('password', {
        hidden: true
      })
    } else {
      form.setSchemaByPath('password', {
        required: false
      })
    }
  }

  const watch = {
    institutionID: val => {
      if (!defaultFormData?.institutionID && val) {
        queryOrganizationalStructureList({
          dataID: val,
          structureType: '1'
        }).then(organizationList => {
          if (organizationList) {
            form.setSchemaByPath('organizationalStructureID', {
              type: 'string',
              widget: 'treeSelect',
              props: {
                treeDefaultExpandAll: true,
                treeData: organizationList.data
              }
            })
          }
        })
      }
    }
  }
  const onFinish = async (formData, errors) => {
    if (errors?.length) return
    try {
      // 执行表单提交
      if (type === ModalType.ADD) {
        await tryAddAccount(formData)
      } else {
        await tryUpdateAccount(formData)
      }
      onVisibleChange(false)
      reload()
    } catch (error) {
      message.error(error)
    }
  }
  return (
    <div>
      {schema && (
        <FormRender
          form={form}
          schema={schema}
          onFinish={onFinish}
          onMount={onMount}
          readOnly={type === ModalType.PREVIEW ? true : false}
          widgets={{ site: SiteInput }}
          watch={watch}
        />
      )}
      <div className="ml-120px">
        {type !== ModalType.PREVIEW && (
          <Button
            type="primary"
            onClick={() => {
              form.submit()
            }}>
            提交
          </Button>
        )}
      </div>
      {/* <Form>
        {schema && (
          <FormRender
            form={form}
            schema={schema}
            onFinish={onFinish}
            onMount={onMount}
            readOnly={type === ModalType.PREVIEW ? true : false}
            widgets={{ site: SiteInput }}
            watch={watch}
          />
        )}
        <div className="ml-120px">
          {type !== ModalType.PREVIEW && (
            <Button
              type="primary"
              onClick={() => {
                form.submit()
              }}>
              提交
            </Button>
          )}
        </div>
      </Form> */}
    </div>
  )
}

export default connect(
  ({
    institutions,
    schemaBase
  }: {
    institutions: InstitutionsModelState
    schemaBase: SchemaBaseModelState
  }) => ({
    accountTypeList: institutions.accountType.map(item => ({
      label: item.name,
      value: item.value
    })),
    // organizationList: institutions.organizationType,
    schema: schemaBase.base[BaseMenuEnum.STAFF]?.schema
  })
)(StaffDrawer)
